"""browser profile

Revision ID: 48fb938c9220
Revises: 700dfebe7c5f
Create Date: 2026-03-02 18:22:46.211722+00:00

"""

from typing import Sequence, Union

import sqlalchemy as sa

from alembic import op

# revision identifiers, used by Alembic.
revision: str = "48fb938c9220"
down_revision: Union[str, None] = "700dfebe7c5f"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("persistent_browser_sessions", sa.Column("browser_profile_id", sa.String(), nullable=True))
    op.create_index(
        op.f("ix_persistent_browser_sessions_browser_profile_id"),
        "persistent_browser_sessions",
        ["browser_profile_id"],
        unique=False,
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_persistent_browser_sessions_browser_profile_id"), table_name="persistent_browser_sessions")
    op.drop_column("persistent_browser_sessions", "browser_profile_id")
    # ### end Alembic commands ###
