Given this form field and context, which option(s) should be selected?

Field label: {{ label }}
Field type: {{ field_type }}

Options:
{% for opt in options %}
{{ loop.index }}. {{ opt }}
{% endfor %}

{% if data %}
Applicant Data:
{% if data is mapping %}
{% for key, value in data.items() %}
- {{ key }}: {{ value }}
{% endfor %}
{% else %}
{{ data }}
{% endif %}
{% endif %}

{% if goal %}
Navigation Goal: {{ goal }}
{% endif %}

Return ONLY a JSON object: {"selected": [1]} for single choice, {"selected": [1, 3]} for multi-select.
Use 1-based indices matching the option numbers above.
For radio buttons (single choice), return exactly one index.
For checkboxes (multi-select), return one or more indices.
